@@ -19,15 +19,15 @@ my $RxMSP = {
 };
 
 my $RxErr = {
-    'filtered' => [
-        qr/Mail Delivery Failed[.]+ User unknown/,
-    ],
-    'mailboxfull' => [
-        qr/The number of messages in recipient's mailbox exceeded the local limit[.]/,
-    ],
+    'filtered' => qr{
+        Mail[ ]Delivery[ ]Failed[.]+[ ]User[ ]unknown
+    }x,
+    'mailboxfull' => qr{
+        The[ ]number[ ]of[ ]messages[ ]in[ ]recipient's[ ]mailbox[ ]exceeded[ ]the[ ]local[ ]limit[.]
+    }x,
 };
 
-sub version     { '4.0.6' }
+sub version     { '4.0.7' }
 sub description { 'BIGLOBE: http://www.biglobe.ne.jp' }
 sub smtpagent   { 'JP::Biglobe' }
 
@@ -49,6 +49,7 @@ sub scan {
     my $rfc822next = { 'from' => 0, 'to' => 0, 'subject' => 0 };
     my $previousfn = '';    # (String) Previous field name
 
+    my $longfields = __PACKAGE__->LONGFIELDS;
     my $stripedtxt = [ split( "\n", $$mbody ) ];
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
 
@@ -68,9 +69,10 @@ sub scan {
                 # Get required headers only
                 my $lhs = $1;
                 my $rhs = $2;
+                my $whs = lc $lhs;
 
                 $previousfn = '';
-                next unless grep { lc( $lhs ) eq lc( $_ ) } @$rfc822head;
+                next unless grep { $whs eq lc( $_ ) } @$rfc822head;
 
                 $previousfn  = $lhs;
                 $rfc822part .= $e."\n";
@@ -78,12 +80,12 @@ sub scan {
             } elsif( $e =~ m/\A[\s\t]+/ ) {
                 # Continued line from the previous line
                 next if $rfc822next->{ lc $previousfn };
-                $rfc822part .= $e."\n" if $previousfn =~ m/\A(?:From|To|Subject)\z/;
+                $rfc822part .= $e."\n" if grep { $previousfn eq $_ } @$longfields;
 
             } else {
                 # Check the end of headers in rfc822 part
-                next unless $previousfn =~ m/\A(?:From|To|Subject)\z/;
-                next unless $e =~ m/\A\z/;
+                next unless grep { $previousfn eq $_ } @$longfields;
+                next if length $e;
                 $rfc822next->{ lc $previousfn } = 1;
             }
 
@@ -155,12 +157,9 @@ sub scan {
 
         SESSION: for my $r ( keys %$RxErr ) {
             # Verify each regular expression of session errors
-            PATTERN: for my $rr ( @{ $RxErr->{ $r } } ) {
-                # Check each regular expression
-                next(PATTERN) unless $e->{'diagnosis'} =~ $rr;
-                $e->{'reason'} = $r;
-                last(SESSION);
-            }
+            next unless $e->{'diagnosis'} =~ $RxErr->{ $r };
+            $e->{'reason'} = $r;
+            last;
         }
 
         $e->{'status'} = Sisimai::RFC3463->getdsn( $e->{'diagnosis'} );
