@@ -23,6 +23,7 @@ my $RxErr = {
     ],
     'filtered' => [
         'RCP-P2',   # The attempted recipient's preferences prevent messages from being delivered.
+        'RCP-P3',   # The attempted recipient's privacy settings blocked the delivery.
     ],
     'mesgtoobig' => [
         'MSG-P1',   # The message exceeds Facebook's maximum allowed size.
@@ -30,7 +31,9 @@ my $RxErr = {
     ],
     'contenterror' => [
         'MSG-P2',   # The message contains an attachment type that Facebook does not accept.
+        'MSG-P3',   # The message contains multiple instances of a header field that can only be present once. Please see RFC 5322, section 3.6 for more information
         'POL-P6',   # The message contains a url that has been blocked by Facebook.
+        'POL-P7',   # The message does not comply with Facebook's abuse policies and will not be accepted.
     ],
     'securityerror' => [
         'POL-P1',   # Your mail server's IP Address is listed on the Spamhaus PBL.
@@ -62,9 +65,12 @@ my $RxErr = {
         'CON-T3',   # Your mail server has opened too many new connections to Facebook's mail servers in a short period of time.
         'CON-T4',   # Your mail server has exceeded the maximum number of recipients for its current connection.
     ],
+    'suspend' => [
+        'RCP-T4',   # The attempted recipient address is currently deactivated. The user may or may not reactivate it.
+    ],
 };
 
-sub version     { '4.0.7' }
+sub version     { '4.0.8' }
 sub description { 'Facebook: https://www.facebook.com' }
 sub smtpagent   { 'US::Facebook' }
 
@@ -86,6 +92,7 @@ sub scan {
     my $rfc822next = { 'from' => 0, 'to' => 0, 'subject' => 0 };
     my $previousfn = '';    # (String) Previous field name
 
+    my $longfields = __PACKAGE__->LONGFIELDS;
     my $stripedtxt = [ split( "\n", $$mbody ) ];
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
     my $fbresponse = '';    # (String) Response code from Facebook
@@ -108,9 +115,10 @@ sub scan {
                 # Get required headers only
                 my $lhs = $1;
                 my $rhs = $2;
+                my $whs = lc $lhs;
 
                 $previousfn = '';
-                next unless grep { lc( $lhs ) eq lc( $_ ) } @$rfc822head;
+                next unless grep { $whs eq lc( $_ ) } @$rfc822head;
 
                 $previousfn  = $lhs;
                 $rfc822part .= $e."\n";
@@ -118,12 +126,12 @@ sub scan {
             } elsif( $e =~ m/\A[\s\t]+/ ) {
                 # Continued line from the previous line
                 next if $rfc822next->{ lc $previousfn };
-                $rfc822part .= $e."\n" if $previousfn =~ m/\A(?:From|To|Subject)\z/;
+                $rfc822part .= $e."\n" if grep { $previousfn eq $_ } @$longfields;
 
             } else {
                 # Check the end of headers in rfc822 part
-                next unless $previousfn =~ m/\A(?:From|To|Subject)\z/;
-                next unless $e =~ m/\A\z/;
+                next unless grep { $previousfn eq $_ } @$longfields;
+                next if length $e;
                 $rfc822next->{ lc $previousfn } = 1;
             }
 
