@@ -8,41 +8,45 @@ use warnings;
 # courier/module.dsn/dsn*.txt
 my $RxMTA = {
     #'from' => qr{Courier mail server at },
-    'begin' => [
-        qr/DELAYS IN DELIVERING YOUR MESSAGE/,
-        qr/UNDELIVERABLE MAIL/,
-    ],
-    'rfc822'  => [
-        qr|\AContent-Type: message/rfc822\z|,
-        qr|\AContent-Type: text/rfc822-headers\z|,
-    ],
-    'endof'  => qr/\A__END_OF_EMAIL_MESSAGE__\z/,
-    'subject' => [
-        qr/NOTICE: mail delivery status[.]/,
-        qr/WARNING: delayed mail[.]/,
-    ],
+    'begin'      => qr{(?:
+         DELAYS[ ]IN[ ]DELIVERING[ ]YOUR[ ]MESSAGE
+        |UNDELIVERABLE[ ]MAIL
+        )
+    }x,
+    'rfc822'     => qr{\AContent-Type:[ ]*(?:
+         message/rfc822
+        |text/rfc822-headers
+        )\z
+    }x,
+    'endof'      => qr/\A__END_OF_EMAIL_MESSAGE__\z/,
+    'subject'    => qr{(?:
+         NOTICE:[ ]mail[ ]delivery[ ]status[.]
+        |WARNING:[ ]delayed[ ]mail[.]
+        )
+    }x,
+    'message-id' => qr/\A[<]courier[.][0-9A-F]+[.]/,
 };
 
 my $RxErr = {
-    'hostunknown' => [
+    'hostunknown' => qr{
         # courier/module.esmtp/esmtpclient.c:526| hard_error(del, ctf, "No such domain.");
-        qr/\ANo such domain[.]\z/,
-    ],
-    'systemerror' => [
+        \ANo[ ]such[ ]domain[.]\z
+    }x,
+    'systemerror' => qr{
         # courier/module.esmtp/esmtpclient.c:531| hard_error(del, ctf,
         # courier/module.esmtp/esmtpclient.c:532|  "This domain's DNS violates RFC 1035.");
-        qr/\AThis domain's DNS violates RFC 1035[.]\z/,
-    ],
+        \AThis[ ]domain's[ ]DNS[ ]violates[ ]RFC[ ]1035[.]\z
+    }x,
 };
 
 my $RxTmp = {
-    'systemerror' => [
+    'systemerror' => qr{
         # courier/module.esmtp/esmtpclient.c:535| soft_error(del, ctf, "DNS lookup failed.");
-        qr/\ADNS lookup failed[.]\z/,
-    ],
+        \ADNS[ ]lookup[ ]failed[.]\z
+    },
 };
 
-sub version     { '4.0.7' }
+sub version     { '4.0.9' }
 sub description { 'Courier MTA' }
 sub smtpagent   { 'Courier' }
 
@@ -54,10 +58,14 @@ sub scan {
     my $class = shift;
     my $mhead = shift // return undef;
     my $mbody = shift // return undef;
+    my $match = 0;
 
-    return undef unless grep { $mhead->{'subject'} =~ $_ } @{ $RxMTA->{'subject'} };
-    # return undef unless( defined $mhead->{'message-id'} );
-    # return undef unless( $mhead->{'message-id'} =~ $RxMTA->{'message-id'} );
+    $match = 1 if $mhead->{'subject'} =~ $RxMTA->{'subject'};
+    if( defined $mhead->{'message-id'} ) {
+        # Message-ID: <courier.4D025E3A.00001792@5jo.example.org>
+        $match = 1 if $mhead->{'message-id'} =~ $RxMTA->{'message-id'};
+    }
+    return undef unless $match;
 
     my $dscontents = [];    # (Ref->Array) SMTP session errors: message/delivery-status
     my $rfc822head = undef; # (Ref->Array) Required header list in message/rfc822 part
@@ -65,6 +73,7 @@ sub scan {
     my $rfc822next = { 'from' => 0, 'to' => 0, 'subject' => 0 };
     my $previousfn = '';    # (String) Previous field name
 
+    my $longfields = __PACKAGE__->LONGFIELDS;
     my $stripedtxt = [ split( "\n", $$mbody ) ];
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
     my $commandtxt = '';    # (String) SMTP Command name begin with the string '>>>'
@@ -82,15 +91,16 @@ sub scan {
 
     for my $e ( @$stripedtxt ) {
         # Read each line between $RxMTA->{'begin'} and $RxMTA->{'rfc822'}.
-        if( ( grep { $e =~ $_ } @{ $RxMTA->{'rfc822'} } ) .. ( $e =~ $RxMTA->{'endof'} ) ) {
+        if( ( $e =~ $RxMTA->{'rfc822'} ) .. ( $e =~ $RxMTA->{'endof'} ) ) {
             # After "message/rfc822"
             if( $e =~ m/\A([-0-9A-Za-z]+?)[:][ ]*(.+)\z/ ) {
                 # Get required headers only
                 my $lhs = $1;
                 my $rhs = $2;
+                my $whs = lc $lhs;
 
                 $previousfn = '';
-                next unless grep { lc( $lhs ) eq lc( $_ ) } @$rfc822head;
+                next unless grep { $whs eq lc( $_ ) } @$rfc822head;
 
                 $previousfn  = $lhs;
                 $rfc822part .= $e."\n";
@@ -98,18 +108,18 @@ sub scan {
             } elsif( $e =~ m/\A[\s\t]+/ ) {
                 # Continued line from the previous line
                 next if $rfc822next->{ lc $previousfn };
-                $rfc822part .= $e."\n" if $previousfn =~ m/\A(?:From|To|Subject)\z/;
+                $rfc822part .= $e."\n" if grep { $previousfn eq $_ } @$longfields;
 
             } else {
                 # Check the end of headers in rfc822 part
-                next unless $previousfn =~ m/\A(?:From|To|Subject)\z/;
-                next unless $e =~ m/\A\z/;
+                next unless grep { $previousfn eq $_ } @$longfields;
+                next if length $e;
                 $rfc822next->{ lc $previousfn } = 1;
             }
 
         } else {
             # Before "message/rfc822"
-            next unless ( grep { $e =~ $_ } @{ $RxMTA->{'begin'} } ) .. ( grep { $e =~ $_ } @{ $RxMTA->{'rfc822'} } );
+            next unless ( $e =~ $RxMTA->{'begin'} ) .. ( $e =~ $RxMTA->{'rfc822'} );
             next unless length $e;
 
             if( $connvalues == scalar( keys %$connheader ) ) {
@@ -242,25 +252,19 @@ sub scan {
         SESSION: while(1) {
             HARD_E: for my $r ( keys %$RxErr ) {
                 # Verify each regular expression of session errors
-                PATTERN: for my $rr ( @{ $RxErr->{ $r } } ) {
-                    # Check each regular expression
-                    next unless $e->{'diagnosis'} =~ $rr;
-                    $e->{'reason'} = $r;
-                    $e->{'softbounce'} = 0;
-                    last(HARD_E);
-                }
+                next unless $e->{'diagnosis'} =~ $RxErr->{ $r };
+                $e->{'reason'} = $r;
+                $e->{'softbounce'} = 0;
+                last(HARD_E);
             }
             last if $e->{'reason'};
 
             SOFT_E: for my $r ( keys %$RxTmp ) {
                 # Verify each regular expression of session errors
-                PATTERN: for my $rr ( @{ $RxErr->{ $r } } ) {
-                    # Check each regular expression
-                    next unless $e->{'diagnosis'} =~ $rr;
-                    $e->{'reason'} = $r;
-                    $e->{'softbounce'} = 1;
-                    last(SOFT_E);
-                }
+                next unless $e->{'diagnosis'} =~ $RxTmp->{ $r };
+                $e->{'reason'} = $r;
+                $e->{'softbounce'} = 1;
+                last(SOFT_E);
             }
 
             last;
