@@ -6,27 +6,31 @@ use warnings;
 
 # Postfix manual - bounce(5) - http://www.postfix.org/bounce.5.html
 my $RxMTA = {
-    'from' => qr/ [(]Mail Delivery System[)]\z/,
-    'begin' => [
-        qr/\A\s+The Postfix program\z/,
-        qr/\A\s+The Postfix on .+ program\z/,   # The Postfix on <os name> program
-        qr/\A\s+The \w+ Postfix program\z/,     # The <name> Postfix program
-        qr/\A\s+The mail system\z/,
-        qr/\AThe \w+ program\z/,                # The <custmized-name> program
-        qr/\AThis is the Postfix program/,
-        qr/\AThis is the \w+ Postfix program/,  # This is the <name> Postfix program
-        qr/\AThis is the \w+ program/,          # This is the <customized-name> Postfix program
-        qr/\AThis is the mail system at host/,  # This is the mail system at host <hostname>.
-    ],
-    'rfc822'  => [
-        qr|\AContent-Type: message/rfc822\z|,
-        qr|\AContent-Type: text/rfc822-headers\z|,
-    ],
+    'from'  => qr/ [(]Mail Delivery System[)]\z/,
+    'begin' => qr{\A(?>
+         [ ]+The[ ](?:
+             Postfix[ ](?:
+                 program\z              # The Postfix program
+                |on[ ].+[ ]program\z    # The Postfix on <os name> program
+                )
+            |\w+[ ]Postfix[ ]program\z  # The <name> Postfix program
+            |mail\ssystem\z             # The mail system
+            |\w+\sprogram\z             # The <custmized-name> program
+            )
+        |This[ ]is[ ]the[ ](?:
+             Postfix[ ]program          # This is the Postfix program
+            |\w+[ ]Postfix[ ]program    # This is the <name> Postfix program
+            |\w+[ ]program              # This is the <customized-name> Postfix program
+            |mail[ ]system[ ]at[ ]host  # This is the mail system at host <hostname>.
+            )
+        )
+    }x,
+    'rfc822'  => qr!\AContent-Type:\s*(?:message/rfc822|text/rfc822-headers)\z!x,
     'endof'   => qr/\A__END_OF_EMAIL_MESSAGE__\z/,
     'subject' => qr/\AUndelivered Mail Returned to Sender\z/,
 };
 
-sub version     { '4.0.8' }
+sub version     { '4.0.10' }
 sub description { 'Postfix' }
 sub smtpagent   { 'Postfix' }
 
@@ -58,6 +62,7 @@ sub scan {
     my $rfc822next = { 'from' => 0, 'to' => 0, 'subject' => 0 };
     my $previousfn = '';    # (String) Previous field name
 
+    my $longfields = __PACKAGE__->LONGFIELDS;
     my $stripedtxt = [ split( "\n", $$mbody ) ];
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
     my $connvalues = 0;     # (Integer) Flag, 1 if all the value of $connheader have been set
@@ -74,16 +79,17 @@ sub scan {
 
     for my $e ( @$stripedtxt ) {
         # Read each line between $RxMTA->{'begin'} and $RxMTA->{'rfc822'}.
-        if( ( grep { $e =~ $_ } @{ $RxMTA->{'rfc822'} } ) .. ( $e =~ $RxMTA->{'endof'} ) ) {
+        if( ( $e =~ $RxMTA->{'rfc822'} ) .. ( $e =~ $RxMTA->{'endof'} ) ) {
             # After "message/rfc822"
 
             if( $e =~ m/\A([-0-9A-Za-z]+?)[:][ ]*(.+)\z/ ) {
                 # Get required headers only
                 my $lhs = $1;
                 my $rhs = $2;
+                my $whs = lc $1;
 
                 $previousfn = '';
-                next unless grep { lc( $lhs ) eq lc( $_ ) } @$rfc822head;
+                next unless grep { $whs eq lc( $_ ) } @$rfc822head;
 
                 $previousfn  = $lhs;
                 $rfc822part .= $e."\n";
@@ -91,18 +97,18 @@ sub scan {
             } elsif( $e =~ m/\A[\s\t]+/ ) {
                 # Continued line from the previous line
                 next if $rfc822next->{ lc $previousfn };
-                $rfc822part .= $e."\n" if $previousfn =~ m/\A(?:From|To|Subject)\z/;
+                $rfc822part .= $e."\n" if grep { $previousfn eq $_ } @$longfields;
 
             } else {
                 # Check the end of headers in rfc822 part
-                next unless $previousfn =~ m/\A(?:From|To|Subject)\z/;
-                next unless $e =~ m/\A\z/;
+                next unless grep { $previousfn eq $_ } @$longfields;
+                next if length $e;
                 $rfc822next->{ lc $previousfn } = 1;
             }
 
         } else {
             # Before "message/rfc822"
-            next unless ( grep { $e =~ $_ } @{ $RxMTA->{'begin'} } ) .. ( grep { $e =~ $_ } @{ $RxMTA->{'rfc822'} } );
+            next unless ( $e =~ $RxMTA->{'begin'} ) .. ( $e =~ $RxMTA->{'rfc822'} );
             next unless length $e;
 
             if( $connvalues == scalar( keys %$connheader ) ) {
@@ -126,7 +132,7 @@ sub scan {
                     $recipients++;
 
                 } elsif( $e =~ m/\AX-Actual-Recipient:[ ]*rfc822;[ ]*([^ ]+)\z/ ||
-                    $e =~ m/\AOriginal-Recipient:[ ]*rfc822;[ ]*([^ ]+)\z/ ) {
+                         $e =~ m/\AOriginal-Recipient:[ ]*rfc822;[ ]*([^ ]+)\z/ ) {
                     # X-Actual-Recipient: RFC822; kijitora@example.co.jp
                     # Original-Recipient: rfc822;kijitora@example.co.jp
                     $v->{'alias'} = $1;
