@@ -9,13 +9,14 @@ my $RxMTA = {
     'begin'    => qr/\AUnable to deliver message to the following address/,
     'endof'    => qr/\A__END_OF_EMAIL_MESSAGE__\z/,
     'rfc822'   => qr/\A--- Original message follows/,
-    'subject'  => [
-        qr/\ADelivery failure/,
-        qr/\Afail(?:ure|ed) delivery/,
-    ],
+    'subject'  => qr{\A(?>
+         Delivery[ ]failure
+        |fail(?:ure|ed)[ ]delivery
+        )
+    }x,
 };
 
-sub version     { '4.0.0' }
+sub version     { '4.0.2' }
 sub description { 'Unknown MTA #2' }
 sub smtpagent   { 'X2' }
 
@@ -28,8 +29,8 @@ sub scan {
     my $mhead = shift // return undef;
     my $mbody = shift // return undef;
 
-    return undef unless $mhead->{'from'} =~ $RxMTA->{'from'};
-    return undef unless grep { $mhead->{'subject'} =~ $_ } @{ $RxMTA->{'subject'} };
+    return undef unless $mhead->{'from'}    =~ $RxMTA->{'from'};
+    return undef unless $mhead->{'subject'} =~ $RxMTA->{'subject'};
 
     my $dscontents = [];    # (Ref->Array) SMTP session errors: message/delivery-status
     my $rfc822head = undef; # (Ref->Array) Required header list in message/rfc822 part
@@ -38,6 +39,7 @@ sub scan {
     my $previousfn = '';    # (String) Previous field name
 
     my $datestring = '';
+    my $longfields = __PACKAGE__->LONGFIELDS;
     my $stripedtxt = [ split( "\n", $$mbody ) ];
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
 
@@ -54,9 +56,10 @@ sub scan {
                 # Get required headers only
                 my $lhs = $1;
                 my $rhs = $2;
+                my $whs = lc $lhs;
 
                 $previousfn = '';
-                next unless grep { lc( $lhs ) eq lc( $_ ) } @$rfc822head;
+                next unless grep { $whs eq lc( $_ ) } @$rfc822head;
 
                 $previousfn  = $lhs;
                 $rfc822part .= $e."\n";
@@ -64,12 +67,12 @@ sub scan {
             } elsif( $e =~ m/\A[\s\t]+/ ) {
                 # Continued line from the previous line
                 next if $rfc822next->{ lc $previousfn };
-                $rfc822part .= $e."\n" if $previousfn =~ m/\A(?:From|To|Subject)\z/;
+                $rfc822part .= $e."\n" if grep { $previousfn eq $_ } @$longfields;
 
             } else {
                 # Check the end of headers in rfc822 part
-                next unless $previousfn =~ m/\A(?:From|To|Subject)\z/;
-                next unless $e =~ m/\A\z/;
+                next unless grep { $previousfn eq $_ } @$longfields;
+                next if length $e;
                 $rfc822next->{ lc $previousfn } = 1;
             }
 
