@@ -5,20 +5,21 @@ use warnings;
 
 # http://tools.ietf.org/html/rfc3464
 my $RxRFC = {
-    'begin'  => [
-        qr|\AContent-Type:\s*message/delivery-status\z|i,
-        qr/\AThe original message was received at /i,
-        qr/\AThis report relates to your message/i,
-    ],
+    'begin'  => qr{\A(?:
+         Content-Type:[ ]*message/delivery-status
+        |The[ ]original[ ]message[ ]was[ ]received[ ]at[ ]
+        |This[ ]report[ ]relates[ ]to[ ]your[ ]message
+        )
+    }xi,
     'endof'  => qr/\A__END_OF_EMAIL_MESSAGE__\z/,
-    'rfc822' => [
-        qr|\AContent-Type:\s*message/rfc822\z|i,
-        qr|\AContent-Type:\s*text/rfc822-headers\z|i,
-        qr|\AReturn-Path:\s*<.+>\z|i,
-    ],
+    'rfc822' => qr{\A(?>
+         Content-Type:[ ]*(?:message/rfc822|text/rfc822-headers)
+        |Return-Path:[ ]*[<].+[>]\z
+        )\z
+    }xi,
 };
 
-sub version     { '4.0.7' };
+sub version     { '4.0.9' };
 sub description { 'Fallback Module for MTAs' };
 sub smtpagent   { 'RFC3464' };
 
@@ -45,6 +46,7 @@ sub scan {
     my $rfc822next = { 'from' => 0, 'to' => 0, 'subject' => 0 };
     my $previousfn = '';    # (String) Previous field name
 
+    my $longfields = Sisimai::MTA->LONGFIELDS;
     my $scannedset = Sisimai::MDA->scan( $mhead, $mbody );
     my $stripedtxt = [ split( "\n", $$mbody ) ];
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
@@ -61,15 +63,16 @@ sub scan {
 
     for my $e ( @$stripedtxt ) {
         # Read each line between $RxRFC->{'begin'} and $RxRFC->{'rfc822'}.
-        if( ( grep { $e =~ $_ } @{ $RxRFC->{'rfc822'} } ) .. ( $e =~ $RxRFC->{'endof'} ) ) {
+        if( ( $e =~ $RxRFC->{'rfc822'} ) .. ( $e =~ $RxRFC->{'endof'} ) ) {
             # After "message/rfc822"
             if( $e =~ m/\A([-0-9A-Za-z]+?)[:][ ]*(.+)\z/ ) {
                 # Get required headers only
                 my $lhs = $1;
                 my $rhs = $2;
+                my $whs = lc $lhs;
 
                 $previousfn = '';
-                next unless grep { lc( $lhs ) eq lc( $_ ) } @$rfc822head;
+                next unless grep { $whs eq lc( $_ ) } @$rfc822head;
 
                 $previousfn  = $lhs;
                 $rfc822part .= $e."\n";
@@ -77,20 +80,18 @@ sub scan {
             } elsif( $e =~ m/\A[\s\t]+/ ) {
                 # Continued line from the previous line
                 next if $rfc822next->{ lc $previousfn };
-                $rfc822part .= $e."\n" if $previousfn =~ m/\A(?:From|To|Subject)\z/;
+                $rfc822part .= $e."\n" if grep { $previousfn eq $_ } @$longfields;
 
             } else {
                 # Check the end of headers in rfc822 part
-                next unless $previousfn =~ m/\A(?:From|To|Subject)\z/;
-                next unless $e =~ m/\A\z/;
+                next unless grep { $previousfn eq $_ } @$longfields;
+                next if length $e;
                 $rfc822next->{ lc $previousfn } = 1;
             }
 
         } else {
             # Before "message/rfc822"
-            next unless 
-                ( grep { $e =~ $_ } @{ $RxRFC->{'begin'} } ) 
-                    .. ( grep { $e =~ $_ } @{ $RxRFC->{'rfc822'} } );
+            next unless ( $e =~ $RxRFC->{'begin'} ) .. ( $e =~ $RxRFC->{'rfc822'} );
             next unless length $e;
   
             $v = $dscontents->[ -1 ];
