@@ -37,7 +37,7 @@ sub is_emailaddress {
     my $class = shift;
     my $email = shift // return 0;
 
-    return 0 if $email =~ m/([\x00-\x1f]|\x1f)/;
+    return 0 if $email =~ m/(?:[\x00-\x1f]|\x1f)/;
     return 1 if $email =~ $Rx->{'ignored'};
     return 0;
 }
@@ -49,7 +49,7 @@ sub is_domainpart {
     my $class = shift;
     my $dpart = shift // return 0;
 
-    return 0 if $dpart =~ m/([\x00-\x1f]|\x1f)/;
+    return 0 if $dpart =~ m/(?:[\x00-\x1f]|\x1f)/;
     return 0 if $dpart =~ m/[@]/;
     return 1 if $dpart =~ $Rx->{'domain'};
     return 0;
@@ -61,14 +61,11 @@ sub is_mailerdaemon {
     # @Return       (Integer) 0 = not mailer-daemon, 1 = mailer-daemon
     my $class = shift;
     my $email = shift // return 0;
-    my $rxmds = [
-        qr/mailer-daemon[@]/i,
-        qr/[<(]mailer-daemon[)>]/i,
-        qr/\Amailer-daemon\z/i,
-        qr/[ ]?mailer-daemon[ ]/i,
-    ];
+    my $rxmds = qr/(?:
+        mailer-daemon[@]|[<(]mailer-daemon[)>]|
+        \Amailer-daemon\z|[ ]?mailer-daemon[ ])/xi;
 
-    return 1 if grep { $email =~ $_ } @$rxmds;
+    return 1 if $email =~ $rxmds;
     return 0;
 }
 
@@ -183,6 +180,7 @@ C<is_emailaddress()> checks the argument is valid email address or not.
 
     print Sisimai::RFC5322->is_emailaddress( 'neko@example.jp' );  # 1
     print Sisimai::RFC5322->is_emailaddress( 'neko%example.jp' );  # 0
+    print Sisimai::RFC5322->is_emailaddress( 'Stray cat <neko@example.jp>' ); # 1
 
 =head2 C<B<is_domainpart( I<Domain> )>>
 
