@@ -7,10 +7,9 @@ sub text  { 'exceedlimit' }
 sub match {
     my $class = shift;
     my $argvs = shift // return undef;
-    my $regex = [
-        qr/message too large/,  # Postfix
-    ];
-    return 1 if grep { lc( $argvs ) =~ $_ } @$regex;
+    my $regex = qr/message too large/;
+
+    return 1 if $argvs =~ $regex;
     return 0;
 }
 
