@@ -40,6 +40,7 @@ my $MonthName = {
         'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
         'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec',
     ],
+    'rx-abbr' => qr/\A(?:Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\z/,
 };
 
 my $DayOfWeek = {
@@ -48,6 +49,7 @@ my $DayOfWeek = {
         'Friday', 'Saturday',
     ],
     'abbr' => [ 'Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', ],
+    'rx-abbr' => qr/\A(?:Sun|Mon|Tue|Wed|Thu|Fri|Sat)\z/,
 };
 
 my $HourName = {
@@ -308,11 +310,11 @@ sub parse {
             # Day of week or Day of week; Thu, Apr, ...
             chop $p if length( $p ) == 4; # Thu, -> Thu
 
-            if( grep { $p eq $_ } @{ $DayOfWeek->{'abbr'} } ) {
+            if( $p =~ $DayOfWeek->{'rx-abbr'} ) {
                 # Day of week; Mon, Thu, Sun,...
                 $v->{'a'} = $p;
 
-            } elsif( grep { $p eq $_ } @{ $MonthName->{'abbr'} } ) {
+            } elsif( $p =~ $MonthName->{'rx-abbr'} ) {
                 # Month name abbr.; Apr, May, ...
                 $v->{'M'} = $p;
             }
